# -*- coding: utf-8 -*-
import stripe


async def auto_paging_iter_patch(self):
    page = self

    while True:
        if (
            "ending_before" in self._retrieve_params
            and "starting_after" not in self._retrieve_params
        ):
            for item in reversed(page):
                yield item
            page = await page.previous_page()
        else:
            for item in page:
                yield item
            page = await page.next_page()

        if page.is_empty:
            break

async def next_page_patch(
    self, api_key=None, stripe_version=None, stripe_account=None, **params
):
    if not self.has_more:
        return self.empty_list(
            api_key=api_key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
        )

    last_id = self.data[-1].id

    params_with_filters = self._retrieve_params.copy()
    params_with_filters.update({"starting_after": last_id})
    params_with_filters.update(params)

    return await self.list(
        api_key=api_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        **params_with_filters
    )

async def previous_page_patch(
    self, api_key=None, stripe_version=None, stripe_account=None, **params
):
    if not self.has_more:
        return self.empty_list(
            api_key=api_key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
        )

    first_id = self.data[0].id

    params_with_filters = self._retrieve_params.copy()
    params_with_filters.update({"ending_before": first_id})
    params_with_filters.update(params)

    return await self.list(
        api_key=api_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        **params_with_filters
    )


stripe.ListObject.auto_paging_iter = auto_paging_iter_patch
stripe.ListObject.previous_page = previous_page_patch
stripe.ListObject.next_page = next_page_patch


# -*- coding: utf-8 -*-
import stripe


def modify_patch(cls, sid, **params):
    raise NotImplementedError(
        "Can't modify a reversal without a transfer ID. "
        "Use stripe.Transfer.modify_reversal('transfer_id', 'reversal_id', ...) "
        "(see https://stripe.com/docs/api/transfer_reversals/update)."
    )


def retrieve_patch(cls, id, api_key=None, **params):
    raise NotImplementedError(
        "Can't retrieve a reversal without a transfer ID. "
        "Use stripe.Transfer.retrieve_reversal('transfer_id', 'reversal_id') "
        "(see https://stripe.com/docs/api/transfer_reversals/retrieve)."
    )


stripe.Reversal.modify = classmethod(modify_patch)
stripe.Reversal.retrieve = classmethod(retrieve_patch)


# -*- coding: utf-8 -*-
import stripe
from async_stripe.api_resources.abstract import patch_nested_resources


async def retrieve_patch(cls, id=None, api_key=None, **params):
    instance = cls(id, api_key, **params)
    await instance.refresh()
    return instance


stripe.Account.retrieve = classmethod(retrieve_patch)


nested_resources = ["capability", "login_link", "external_account", "person",]

patch_nested_resources(stripe.Account, nested_resources)



from stripe.api_resources.abstract.searchable_api_resource import (
    SearchableAPIResource,
)
from stripe.api_resources.search_result_object import SearchResultObject
from typing import Awaitable


def _search_patch(
    cls,
    search_url,
    api_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
) -> Awaitable[SearchResultObject]:
    ret = cls._static_request(
        "get",
        search_url,
        api_key=api_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        params=params,
    )
    if not isinstance(ret, Awaitable):
        raise TypeError(
            "Expected awaitable object from API, got %s"
            % (type(ret).__name__,)
        )

    return ret


SearchableAPIResource._search = classmethod(_search_patch)


from stripe.api_resources.abstract.listable_api_resource import (
    ListableAPIResource,
)
from stripe.api_resources.list_object import ListObject
from typing import Awaitable


async def auto_paging_iter_patch(cls, *args, **params):
    obj = await cls.list(*args, **params)
    return obj.auto_paging_iter()


def list_patch(
    cls, api_key=None, stripe_version=None, stripe_account=None, **params
) -> Awaitable[ListObject]:
    result = cls._static_request(
        "get",
        cls.class_url(),
        api_key=api_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        params=params,
    )

    if not isinstance(result, Awaitable):
        raise TypeError(
            "Expected awaitable object from API, got %s"
            % (type(result).__name__,)
        )

    return result


ListableAPIResource.auto_paging_iter = classmethod(auto_paging_iter_patch)
ListableAPIResource.list = classmethod(list_patch)


for subclass in ListableAPIResource.__subclasses__():
    subclass.auto_paging_iter = classmethod(auto_paging_iter_patch)



# -*- coding: utf-8 -*-
import stripe


async def modify_patch(cls, sid, **params):
    raise NotImplementedError(
        "Can't modify a bank account without a customer or account ID. "
        "Use stripe.Customer.modify_source('customer_id', 'bank_account_id', ...) "
        "(see https://stripe.com/docs/api/customer_bank_accounts/update) or "
        "stripe.Account.modify_external_account('customer_id', 'bank_account_id', ...) "
        "(see https://stripe.com/docs/api/external_account_bank_accounts/update)."
    )


async def retrieve_patch(
    cls,
    id,
    api_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    raise NotImplementedError(
        "Can't retrieve a bank account without a customer or account ID. "
        "Use stripe.customer.retrieve_source('customer_id', 'bank_account_id') "
        "(see https://stripe.com/docs/api/customer_bank_accounts/retrieve) or "
        "stripe.Account.retrieve_external_account('account_id', 'bank_account_id') "
        "(see https://stripe.com/docs/api/external_account_bank_accounts/retrieve)."
    )


stripe.BankAccount.modify = classmethod(modify_patch)
stripe.BankAccount.retrieve = classmethod(retrieve_patch)


# -*- coding: utf-8 -*-
import stripe


async def auto_paging_iter_patch(self):
    page = self

    while True:
        for item in page:
            yield item
        page = await page.next_search_result_page()

        if page.is_empty:
            break


async def next_search_result_page_patch(
    self, api_key=None, stripe_version=None, stripe_account=None, **params
):
    if not self.has_more:
        return self.empty_search_result(
            api_key=api_key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
        )

    params_with_filters = self._retrieve_params.copy()
    params_with_filters.update({"page": self.next_page})
    params_with_filters.update(params)

    result = await self.search(
        api_key=api_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        **params_with_filters
    )
    assert isinstance(result, stripe.SearchResultObject)
    return result


stripe.SearchResultObject.auto_paging_iter = auto_paging_iter_patch
stripe.SearchResultObject.next_search_result_page = next_search_result_page_patch



from stripe import util
from stripe import api_requestor
from stripe.stripe_object import StripeObject
from stripe.api_resources.abstract.api_resource import APIResource


async def _static_request_patch(
    cls,
    method_,
    url_,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    params=None
):
    params = None if params is None else params.copy()
    api_key = util.read_special_variable(params, "api_key", api_key)
    idempotency_key = util.read_special_variable(
        params, "idempotency_key", idempotency_key
    )
    stripe_version = util.read_special_variable(
        params, "stripe_version", stripe_version
    )
    stripe_account = util.read_special_variable(
        params, "stripe_account", stripe_account
    )
    headers = util.read_special_variable(params, "headers", None)

    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )

    if idempotency_key is not None:
        headers = {} if headers is None else headers.copy()
        headers.update(util.populate_headers(idempotency_key))

    response, api_key = await requestor.request(method_, url_, params, headers)
    return util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account, params
    )

APIResource._static_request = classmethod(_static_request_patch)


async def _static_request_stream_patch(
    cls,
    method_,
    url_,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    params=None
):
    params = None if params is None else params.copy()
    api_key = util.read_special_variable(params, "api_key", api_key)
    idempotency_key = util.read_special_variable(
        params, "idempotency_key", idempotency_key
    )
    stripe_version = util.read_special_variable(
        params, "stripe_version", stripe_version
    )
    stripe_account = util.read_special_variable(
        params, "stripe_account", stripe_account
    )
    headers = util.read_special_variable(params, "headers", None)

    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )

    if idempotency_key is not None:
        headers = {} if headers is None else headers.copy()
        headers.update(util.populate_headers(idempotency_key))

    response, _ = await requestor.request_stream(method_, url_, params, headers)
    return response

APIResource._static_request_stream = classmethod(_static_request_stream_patch)


async def retrieve_patch(cls, id, api_key=None, **params):
    instance = cls(id, api_key, **params)
    await instance.refresh()
    return instance

APIResource.retrieve = classmethod(retrieve_patch)


async def _request_patch(
    self,
    method_,
    url_,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    headers=None,
    params=None,
):
    obj = await StripeObject._request(
        self,
        method_,
        url_,
        api_key,
        idempotency_key,
        stripe_version,
        stripe_account,
        headers,
        params,
    )

    if type(self) is type(obj):
        self.refresh_from(obj)
        return self
    else:
        return obj

APIResource._request = _request_patch


async def _request_and_refresh_patch(
    self,
    method_,
    url_,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    headers=None,
    params=None,
):
    obj = await StripeObject._request(
        self,
        method_,
        url_,
        api_key,
        idempotency_key,
        stripe_version,
        stripe_account,
        headers,
        params,
    )

    self.refresh_from(obj)
    return self

APIResource._request_and_refresh = _request_and_refresh_patch



# -*- coding: utf-8 -*-
import stripe
from stripe import util
from stripe.api_resources.abstract import ListableAPIResource
from async_stripe.api_resources.abstract import patch_nested_resources


nested_resources = ["refund"]

patch_nested_resources(stripe.ApplicationFee, nested_resources)


from stripe import util
from stripe.stripe_object import StripeObject
from stripe import api_requestor


async def _request_patch(
    self,
    method_,
    url_,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    headers=None,
    params=None,
):
    params = None if params is None else params.copy()
    api_key = util.read_special_variable(params, "api_key", api_key)
    idempotency_key = util.read_special_variable(
        params, "idempotency_key", idempotency_key
    )
    stripe_version = util.read_special_variable(
        params, "stripe_version", stripe_version
    )
    stripe_account = util.read_special_variable(
        params, "stripe_account", stripe_account
    )
    headers = util.read_special_variable(params, "headers", headers)
        
    stripe_account = stripe_account or self.stripe_account
    stripe_version = stripe_version or self.stripe_version
    api_key = api_key or self.api_key
    params = params or self._retrieve_params

    requestor = api_requestor.APIRequestor(
        key=api_key,
        api_base=self.api_base(),
        api_version=stripe_version,
        account=stripe_account,
    )

    if idempotency_key is not None:
        headers = {} if headers is None else headers.copy()
        headers.update(util.populate_headers(idempotency_key))

    response, api_key = await requestor.request(method_, url_, params, headers)

    return util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account, params
    )

StripeObject._request = _request_patch


async def request_stream_patch(self, method, url, params=None, headers=None):
    if params is None:
        params = self._retrieve_params
    requestor = api_requestor.APIRequestor(
        key=self.api_key,
        api_base=self.api_base(),
        api_version=self.stripe_version,
        account=self.stripe_account,
    )
    response, _ = await requestor.request_stream(method, url, params, headers)

    return response

StripeObject.request_stream = request_stream_patch



# -*- coding: utf-8 -*-
import stripe


async def retrieve_patch(cls, id, api_key=None, **params):
    raise NotImplementedError(
        "Can't retrieve a refund without an application fee ID. "
        "Use application_fee.refunds.retrieve('refund_id') instead."
    )


stripe.ApplicationFeeRefund.retrieve = classmethod(retrieve_patch)



# -*- coding: utf-8 -*-
import stripe


async def modify_patch(cls, sid, **params):
    raise NotImplementedError(
        "Can't modify a person without an account ID. "
        "Use stripe.Account.modify_person('account_id', 'person_id', ...) "
        "(see https://stripe.com/docs/api/persons/update)."
    )


async def retrieve_patch(cls, id, api_key=None, **params):
    raise NotImplementedError(
        "Can't retrieve a person without an account ID. "
        "Use stripe.Account.retrieve_person('account_id', 'person_id') "
        "(see https://stripe.com/docs/api/persons/retrieve)."
    )


stripe.Person.modify = classmethod(modify_patch)
stripe.Person.retrieve = classmethod(retrieve_patch)


# -*- coding: utf-8 -*-
import stripe
from async_stripe.api_resources.abstract import patch_nested_resources
from stripe import util


netsted_resources = ['balance_transaction', 'cash_balance_transaction', 'source', 'tax_id']
patch_nested_resources(stripe.Customer, netsted_resources)



from stripe import util
from stripe.api_resources.abstract.updateable_api_resource import (
    UpdateableAPIResource,
)


async def save_patch(self, idempotency_key=None):
    """
    The `save` method is deprecated and will be removed in a future major version of the library.
    Use the class method `modify` on the resource instead.
    """
    updated_params = self.serialize(None)

    if updated_params:
        await self._request_and_refresh(
            "post",
            self.instance_url(),
            idempotency_key=idempotency_key,
            params=updated_params,
        )
    else:
        util.logger.debug("Trying to save already saved object %r", self)
    return self


UpdateableAPIResource.save = save_patch


for subclass in UpdateableAPIResource.__subclasses__():
    subclass.save = save_patch


from stripe.api_resources.abstract import APIResource


def patch_nested_resources(cls, nested_resources):
    for resource in nested_resources:
        nested_resource_request_method = '%ss_request' % resource

        async def nested_resource_request(
            cls,
            method,
            url,
            api_key=None,
            idempotency_key=None,
            stripe_version=None,
            stripe_account=None,
            **params
        ):
            return await APIResource._static_request(
                method,
                url,
                api_key=api_key,
                idempotency_key=idempotency_key,
                stripe_version=stripe_version,
                stripe_account=stripe_account,
                params=params,
            )
    
        setattr(cls, nested_resource_request_method, classmethod(nested_resource_request))


# -*- coding: utf-8 -*-
import stripe
from stripe import util
from async_stripe.api_resources.abstract import patch_nested_resources


netsted_resources = ["reversal"]
patch_nested_resources(stripe.Transfer, netsted_resources)



# -*- coding: utf-8 -*-
import stripe
from stripe import util
from stripe import api_requestor
from urllib.parse import quote_plus


async def _cls_pdf_patch(
    cls,
    sid,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    url = "%s/%s/%s" % (
        cls.class_url(),
        quote_plus(sid),
        "pdf",
    )
    requestor = api_requestor.APIRequestor(
        api_key,
        api_base=stripe.upload_api_base,
        api_version=stripe_version,
        account=stripe_account,
    )
    headers = util.populate_headers(idempotency_key)
    response, _ = await requestor.request_stream("get", url, params, headers)
    return response

@util.class_method_variant("_cls_pdf")
async def pdf_patch(
    self,
    api_key=None,
    api_version=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    version = api_version or stripe_version
    requestor = api_requestor.APIRequestor(
        api_key,
        api_base=stripe.upload_api_base,
        api_version=version,
        account=stripe_account,
    )
    url = self.instance_url() + "/pdf"
    return await requestor.request_stream("get", url, params=params)


stripe.Quote._cls_pdf = classmethod(_cls_pdf_patch)
stripe.Quote.pdf = pdf_patch



# -*- coding: utf-8 -*-
from async_stripe.api_resources.abstract import *

from async_stripe.api_resources import list_object
from async_stripe.api_resources import search_result_object


from async_stripe.api_resources import account
from async_stripe.api_resources import application_fee
from async_stripe.api_resources import application_fee_refund
from async_stripe.api_resources import bank_account
from async_stripe.api_resources import capability
from async_stripe.api_resources import card
from async_stripe.api_resources import charge
from async_stripe.api_resources import customer
from async_stripe.api_resources import ephermal_key
from async_stripe.api_resources import file
from async_stripe.api_resources import person
from async_stripe.api_resources import quote
from async_stripe.api_resources import reversal
from async_stripe.api_resources import source
from async_stripe.api_resources import transfer
from async_stripe.api_resources import usage_record



# -*- coding: utf-8 -*-
import stripe
from stripe import util


async def mark_as_fraudulent_patch(self, idempotency_key=None):
    params = {"fraud_details": {"user_report": "fraudulent"}}
    url = self.instance_url()
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def mark_as_safe_patch(self, idempotency_key=None):
    params = {"fraud_details": {"user_report": "safe"}}
    url = self.instance_url()
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.Charge.mark_as_fraudulent = mark_as_fraudulent_patch
stripe.Charge.mark_as_safe = mark_as_safe_patch
